@@ -5,8 +5,11 @@
  | program/include/html.php                                              |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2010, Roundcube Dev, - Switzerland                 |
- | Licensed under the GNU GPL                                            |
+ | Copyright (C) 2005-2011, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Licensed under the GNU General Public License version 3 or            |
+ | any later version with exceptions for skins & plugins.                |
+ | See the README file for a full license statement.                     |
  |                                                                       |
  | PURPOSE:                                                              |
  |   Helper class to create valid XHTML code                             |
@@ -15,7 +18,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: html.php 3989 2010-09-25 13:03:53Z alec $
+ $Id$
 
  */
 
@@ -32,6 +35,7 @@ class html
     protected $allowed = array();
     protected $content;
 
+    public static $doctype = 'xhtml';
     public static $lc_tags = true;
     public static $common_attrib = array('id','class','style','title','align');
     public static $containers = array('iframe','div','span','p','h1','h2','h3','form','textarea','table','thead','tbody','tr','th','td','style','script');
@@ -39,7 +43,7 @@ class html
     /**
      * Constructor
      *
-     * @param array Hash array with tag attributes
+     * @param array $attrib Hash array with tag attributes
      */
     public function __construct($attrib = array())
     {
@@ -63,33 +67,56 @@ class html
     /**
      * Generic method to create a HTML tag
      *
-     * @param string Tag name
-     * @param array  Tag attributes as key/value pairs
-     * @param string Optinal Tag content (creates a container tag)
-     * @param array  List with allowed attributes, omit to allow all
+     * @param string $tagname Tag name
+     * @param array  $attrib  Tag attributes as key/value pairs
+     * @param string $content Optinal Tag content (creates a container tag)
+     * @param array  $allowed_attrib List with allowed attributes, omit to allow all
      * @return string The XHTML tag
      */
     public static function tag($tagname, $attrib = array(), $content = null, $allowed_attrib = null)
     {
+        if (is_string($attrib))
+            $attrib = array('class' => $attrib);
+
         $inline_tags = array('a','span','img');
         $suffix = $attrib['nl'] || ($content && $attrib['nl'] !== false && !in_array($tagname, $inline_tags)) ? "\n" : '';
 
         $tagname = self::$lc_tags ? strtolower($tagname) : $tagname;
-        if ($content || in_array($tagname, self::$containers)) {
-            $templ = $attrib['noclose'] ? "<%s%s>%s" : "<%s%s>%s</%s>%s";
-            unset($attrib['noclose']);
-            return sprintf($templ, $tagname, self::attrib_string($attrib, $allowed_attrib), $content, $tagname, $suffix);
+        if (isset($content) || in_array($tagname, self::$containers)) {
+            $suffix = $attrib['noclose'] ? $suffix : '</' . $tagname . '>' . $suffix;
+            unset($attrib['noclose'], $attrib['nl']);
+            return '<' . $tagname  . self::attrib_string($attrib, $allowed_attrib) . '>' . $content . $suffix;
         }
         else {
-            return sprintf("<%s%s />%s", $tagname, self::attrib_string($attrib, $allowed_attrib), $suffix);
+            return '<' . $tagname  . self::attrib_string($attrib, $allowed_attrib) . '>' . $suffix;
         }
     }
 
+    /**
+     *
+     */
+    public static function doctype($type)
+    {
+        $doctypes = array(
+            'html5'        => '<!DOCTYPE html>',
+            'xhtml'        => '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">',
+            'xhtml-trans'  => '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">',
+            'xhtml-strict' => '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">',
+        );
+
+        if ($doctypes[$type]) {
+            self::$doctype = preg_replace('/-\w+$/', '', $type);
+            return $doctypes[$type];
+        }
+
+        return '';
+    }
+
     /**
      * Derrived method for <div> containers
      *
-     * @param mixed  Hash array with tag attributes or string with class name
-     * @param string Div content
+     * @param mixed  $attr Hash array with tag attributes or string with class name
+     * @param string $cont Div content
      * @return string HTML code
      * @see html::tag()
      */
@@ -104,8 +131,8 @@ class html
     /**
      * Derrived method for <p> blocks
      *
-     * @param mixed  Hash array with tag attributes or string with class name
-     * @param string Paragraph content
+     * @param mixed  $attr Hash array with tag attributes or string with class name
+     * @param string $cont Paragraph content
      * @return string HTML code
      * @see html::tag()
      */
@@ -120,7 +147,7 @@ class html
     /**
      * Derrived method to create <img />
      *
-     * @param mixed Hash array with tag attributes or string with image source (src)
+     * @param mixed $attr Hash array with tag attributes or string with image source (src)
      * @return string HTML code
      * @see html::tag()
      */
@@ -130,14 +157,14 @@ class html
             $attr = array('src' => $attr);
         }
         return self::tag('img', $attr + array('alt' => ''), null, array_merge(self::$common_attrib,
-	    array('src','alt','width','height','border','usemap')));
+	        array('src','alt','width','height','border','usemap','onclick')));
     }
 
     /**
      * Derrived method for link tags
      *
-     * @param mixed  Hash array with tag attributes or string with link location (href)
-     * @param string Link content
+     * @param mixed  $attr Hash array with tag attributes or string with link location (href)
+     * @param string $cont Link content
      * @return string HTML code
      * @see html::tag()
      */
@@ -147,14 +174,14 @@ class html
             $attr = array('href' => $attr);
         }
         return self::tag('a', $attr, $cont, array_merge(self::$common_attrib,
-	    array('href','target','name','onclick','onmouseover','onmouseout','onmousedown','onmouseup')));
+	    array('href','target','name','rel','onclick','onmouseover','onmouseout','onmousedown','onmouseup')));
     }
 
     /**
      * Derrived method for inline span tags
      *
-     * @param mixed  Hash array with tag attributes or string with class name
-     * @param string Tag content
+     * @param mixed  $attr Hash array with tag attributes or string with class name
+     * @param string $cont Tag content
      * @return string HTML code
      * @see html::tag()
      */
@@ -169,8 +196,8 @@ class html
     /**
      * Derrived method for form element labels
      *
-     * @param mixed  Hash array with tag attributes or string with 'for' attrib
-     * @param string Tag content
+     * @param mixed  $attr Hash array with tag attributes or string with 'for' attrib
+     * @param string $cont Tag content
      * @return string HTML code
      * @see html::tag()
      */
@@ -185,7 +212,7 @@ class html
     /**
      * Derrived method to create <iframe></iframe>
      *
-     * @param mixed Hash array with tag attributes or string with frame source (src)
+     * @param mixed $attr Hash array with tag attributes or string with frame source (src)
      * @return string HTML code
      * @see html::tag()
      */
@@ -195,7 +222,31 @@ class html
             $attr = array('src' => $attr);
         }
         return self::tag('iframe', $attr, $cont, array_merge(self::$common_attrib,
-	    array('src','name','width','height','border','frameborder')));
+            array('src','name','width','height','border','frameborder')));
+    }
+
+    /**
+     * Derrived method to create <script> tags
+     *
+     * @param mixed $attr Hash array with tag attributes or string with script source (src)
+     * @param string $cont Javascript code to be placed as tag content
+     * @return string HTML code
+     * @see html::tag()
+     */
+    public static function script($attr, $cont = null)
+    {
+        if (is_string($attr)) {
+            $attr = array('src' => $attr);
+        }
+        if ($cont) {
+            if (self::$doctype == 'xhtml')
+                $cont = "\n/* <![CDATA[ */\n" . $cont . "\n/* ]]> */\n";
+            else
+                $cont = "\n" . $cont . "\n";
+        }
+
+        return self::tag('script', $attr + array('type' => 'text/javascript', 'nl' => true),
+            $cont, array_merge(self::$common_attrib, array('src','type','charset')));
     }
 
     /**
@@ -212,8 +263,8 @@ class html
     /**
      * Create string with attributes
      *
-     * @param array Associative arry with tag attributes
-     * @param array List of allowed attributes
+     * @param array $attrib Associative arry with tag attributes
+     * @param array $allowed List of allowed attributes
      * @return string Valid attribute string
      */
     public static function attrib_string($attrib = array(), $allowed = null)
@@ -243,14 +294,14 @@ class html
             // attributes with no value
             if (in_array($key, array('checked', 'multiple', 'disabled', 'selected'))) {
                 if ($value) {
-                    $attrib_arr[] = sprintf('%s="%s"', $key, $key);
+                    $attrib_arr[] = $key . '="' . $key . '"';
                 }
             }
             else if ($key=='value') {
-                $attrib_arr[] = sprintf('%s="%s"', $key, Q($value, 'strict', false));
+                $attrib_arr[] = $key . '="' . Q($value, 'strict', false) . '"';
             }
             else {
-                $attrib_arr[] = sprintf('%s="%s"', $key, Q($value));
+                $attrib_arr[] = $key . '="' . Q($value) . '"';
             }
         }
         return count($attrib_arr) ? ' '.implode(' ', $attrib_arr) : '';
@@ -267,9 +318,14 @@ class html_inputfield extends html
     protected $tagname = 'input';
     protected $type = 'text';
     protected $allowed = array('type','name','value','size','tabindex',
-	'autocomplete','checked','onchange','onclick','disabled','readonly',
-	'spellcheck','results','maxlength','src');
+        'autocomplete','checked','onchange','onclick','disabled','readonly',
+        'spellcheck','results','maxlength','src','multiple','placeholder');
 
+    /**
+     * Object constructor
+     *
+     * @param array $attrib Associative array with tag attributes
+     */
     public function __construct($attrib = array())
     {
         if (is_array($attrib)) {
@@ -279,17 +335,13 @@ class html_inputfield extends html
         if ($attrib['type']) {
             $this->type = $attrib['type'];
         }
-
-        if ($attrib['newline']) {
-            $this->newline = true;
-        }
     }
 
     /**
      * Compose input tag
      *
-     * @param string Field value
-     * @param array Additional attributes to override
+     * @param string $value Field value
+     * @param array  $attrib Additional attributes to override
      * @return string HTML output
      */
     public function show($value = null, $attrib = null)
@@ -325,16 +377,17 @@ class html_passwordfield extends html_inputfield
  * @package HTML
  */
 
-class html_hiddenfield extends html_inputfield
+class html_hiddenfield extends html
 {
+    protected $tagname = 'input';
     protected $type = 'hidden';
     protected $fields_arr = array();
-    protected $newline = true;
+    protected $allowed = array('type','name','value','onchange','disabled','readonly');
 
     /**
      * Constructor
      *
-     * @param array Named tag attributes
+     * @param array $attrib Named tag attributes
      */
     public function __construct($attrib = null)
     {
@@ -346,7 +399,7 @@ class html_hiddenfield extends html_inputfield
     /**
      * Add a hidden field to this instance
      *
-     * @param array Named tag attributes
+     * @param array $attrib Named tag attributes
      */
     public function add($attrib)
     {
@@ -380,8 +433,8 @@ class html_radiobutton extends html_inputfield
     /**
      * Get HTML code for this object
      *
-     * @param string Value of the checked field
-     * @param array Additional attributes to override
+     * @param string $value  Value of the checked field
+     * @param array  $attrib Additional attributes to override
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -410,8 +463,8 @@ class html_checkbox extends html_inputfield
     /**
      * Get HTML code for this object
      *
-     * @param string Value of the checked field
-     * @param array Additional attributes to override
+     * @param string $value  Value of the checked field
+     * @param array  $attrib Additional attributes to override
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -442,8 +495,8 @@ class html_textarea extends html
     /**
      * Get HTML code for this object
      *
-     * @param string Textbox value
-     * @param array Additional attributes to override
+     * @param string $value  Textbox value
+     * @param array  $attrib Additional attributes to override
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -496,13 +549,13 @@ class html_select extends html
     protected $tagname = 'select';
     protected $options = array();
     protected $allowed = array('name','size','tabindex','autocomplete',
-	'multiple','onchange','disabled');
+	'multiple','onchange','disabled','rel');
     
     /**
      * Add a new option to this drop-down
      *
-     * @param mixed Option name or array with option names
-     * @param mixed Option value or array with option values
+     * @param mixed $names  Option name or array with option names
+     * @param mixed $values Option value or array with option values
      */
     public function add($names, $values = null)
     {
@@ -516,12 +569,11 @@ class html_select extends html
         }
     }
 
-
     /**
      * Get HTML code for this object
      *
-     * @param string Value of the selection option
-     * @param array Additional attributes to override
+     * @param string $select Value of the selection option
+     * @param array  $attrib Additional attributes to override
      * @return string HTML output
      */
     public function show($select = array(), $attrib = null)
@@ -555,24 +607,29 @@ class html_table extends html
 {
     protected $tagname = 'table';
     protected $allowed = array('id','class','style','width','summary',
-	'cellpadding','cellspacing','border');
+	    'cellpadding','cellspacing','border');
 
     private $header = array();
     private $rows = array();
     private $rowindex = 0;
     private $colindex = 0;
 
-
+    /**
+     * Constructor
+     *
+     * @param array $attrib Named tag attributes
+     */
     public function __construct($attrib = array())
     {
-        $this->attrib = array_merge($attrib, array('summary' => '', 'border' => 0));
+        $default_attrib = self::$doctype == 'xhtml' ? array('summary' => '', 'border' => 0) : array();
+        $this->attrib = array_merge($attrib, $default_attrib);
     }
 
     /**
      * Add a table cell
      *
-     * @param array Cell attributes
-     * @param string Cell content
+     * @param array  $attr Cell attributes
+     * @param string $cont Cell content
      */
     public function add($attr, $cont)
     {
@@ -595,8 +652,8 @@ class html_table extends html
     /**
      * Add a table header cell
      *
-     * @param array Cell attributes
-     * @param string Cell content
+     * @param array  $attr Cell attributes
+     * @param string $cont Cell content
      */
     public function add_header($attr, $cont)
     {
@@ -618,17 +675,17 @@ class html_table extends html
     public function remove_column($class)
     {
         // Remove the header
-        foreach($this->header as $index=>$header){
-            if($header->attrib['class'] == $class){
+        foreach ($this->header as $index=>$header){
+            if ($header->attrib['class'] == $class){
                 unset($this->header[$index]);
                 break;
             }
         }
 
         // Remove cells from rows
-        foreach($this->rows as $i=>$row){
-            foreach($row->cells as $j=>$cell){
-                if($cell->attrib['class'] == $class){
+        foreach ($this->rows as $i=>$row){
+            foreach ($row->cells as $j=>$cell){
+                if ($cell->attrib['class'] == $class){
                     unset($this->rows[$i]->cells[$j]);
                     break;
                 }
@@ -636,11 +693,10 @@ class html_table extends html
         }
     }
 
-
     /**
      * Jump to next row
      *
-     * @param array Row attributes
+     * @param array $attr Row attributes
      */
     public function add_row($attr = array())
     {
@@ -652,29 +708,48 @@ class html_table extends html
     }
 
     /**
-     * Set current row attrib
+     * Set row attributes
      *
-     * @param array Row attributes
+     * @param array $attr  Row attributes
+     * @param int   $index Optional row index (default current row index)
      */
-    public function set_row_attribs($attr = array())
+    public function set_row_attribs($attr = array(), $index = null)
     {
         if (is_string($attr))
     	    $attr = array('class' => $attr);
 
-        $this->rows[$this->rowindex]->attrib = $attr;
+        if ($index === null)
+            $index = $this->rowindex;
+
+        $this->rows[$index]->attrib = $attr;
+    }
+
+    /**
+     * Get row attributes
+     *
+     * @param int $index Row index
+     *
+     * @return array Row attributes
+     */
+    public function get_row_attribs($index = null)
+    {
+        if ($index === null)
+            $index = $this->rowindex;
+
+        return $this->rows[$index] ? $this->rows[$index]->attrib : null;
     }
 
     /**
      * Build HTML output of the table data
      *
-     * @param array Table attributes
+     * @param array $attrib Table attributes
      * @return string The final table HTML code
      */
     public function show($attrib = null)
     {
         if (is_array($attrib))
             $this->attrib = array_merge($this->attrib, $attrib);
-        
+
         $thead = $tbody = "";
 
         // include <thead>
@@ -683,7 +758,7 @@ class html_table extends html
             foreach ($this->header as $c => $col) {
                 $rowcontent .= self::tag('td', $col->attrib, $col->content);
             }
-            $thead = self::tag('thead', null, self::tag('tr', null, $rowcontent));
+            $thead = self::tag('thead', null, self::tag('tr', null, $rowcontent, parent::$common_attrib));
         }
 
         foreach ($this->rows as $r => $row) {
@@ -693,7 +768,7 @@ class html_table extends html
             }
 
             if ($r < $this->rowindex || count($row->cells)) {
-                $tbody .= self::tag('tr', $row->attrib, $rowcontent);
+                $tbody .= self::tag('tr', $row->attrib, $rowcontent, parent::$common_attrib);
             }
         }
 
@@ -707,7 +782,7 @@ class html_table extends html
         unset($this->attrib['cols'], $this->attrib['rowsonly']);
         return parent::show();
     }
-    
+
     /**
      * Count number of rows
      *
@@ -717,5 +792,15 @@ class html_table extends html
     {
       return count($this->rows);
     }
+
+    /**
+     * Remove table body (all rows)
+     */
+    public function remove_body()
+    {
+        $this->rows     = array();
+        $this->rowindex = 0;
+    }
+
 }
 
