@@ -5,8 +5,11 @@
  | program/include/rcube_config.php                                      |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2010, Roundcube Dev. - Switzerland                 |
- | Licensed under the GNU GPL                                            |
+ | Copyright (C) 2008-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Licensed under the GNU General Public License version 3 or            |
+ | any later version with exceptions for skins & plugins.                |
+ | See the README file for a full license statement.                     |
  |                                                                       |
  | PURPOSE:                                                              |
  |   Class to read configuration settings                                |
@@ -15,7 +18,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_config.php 3989 2010-09-25 13:03:53Z alec $
+ $Id$
 
 */
 
@@ -30,6 +33,18 @@ class rcube_config
     private $errors = array();
     private $userprefs = array();
 
+    /**
+     * Renamed options
+     *
+     * @var array
+     */
+    private $legacy_props = array(
+        // new name => old name
+        'default_folders'      => 'default_imap_folders',
+        'mail_pagesize'        => 'pagesize',
+        'addressbook_pagesize' => 'pagesize',
+    );
+
 
     /**
      * Object constructor
@@ -47,10 +62,6 @@ class rcube_config
      */
     private function load()
     {
-        // start output buffering, we don't need any output yet, 
-        // it'll be cleared after reading of config files, etc.
-        ob_start();
-    
         // load main config file
         if (!$this->load_from_file(RCMAIL_CONFIG_DIR . '/main.inc.php'))
             $this->errors[] = 'main.inc.php was not found.';
@@ -58,7 +69,7 @@ class rcube_config
         // load database config
         if (!$this->load_from_file(RCMAIL_CONFIG_DIR . '/db.inc.php'))
             $this->errors[] = 'db.inc.php was not found.';
-    
+
         // load host-specific configuration
         $this->load_host_config();
 
@@ -66,19 +77,23 @@ class rcube_config
         if (empty($this->prop['skin']) && !empty($this->prop['skin_path']))
             $this->prop['skin'] = str_replace('skins/', '', unslashify($this->prop['skin_path']));
         else if (empty($this->prop['skin']))
-            $this->prop['skin'] = 'default';
+            $this->prop['skin'] = 'larry';
+
+        // larry is the new default skin :-)
+        if ($this->prop['skin'] == 'default')
+            $this->prop['skin'] = 'larry';
 
         // fix paths
         $this->prop['log_dir'] = $this->prop['log_dir'] ? realpath(unslashify($this->prop['log_dir'])) : INSTALL_PATH . 'logs';
         $this->prop['temp_dir'] = $this->prop['temp_dir'] ? realpath(unslashify($this->prop['temp_dir'])) : INSTALL_PATH . 'temp';
-    
+
         // fix default imap folders encoding
         foreach (array('drafts_mbox', 'junk_mbox', 'sent_mbox', 'trash_mbox') as $folder)
             $this->prop[$folder] = rcube_charset_convert($this->prop[$folder], RCMAIL_CHARSET, 'UTF7-IMAP');
 
-        if (!empty($this->prop['default_imap_folders']))
-            foreach ($this->prop['default_imap_folders'] as $n => $folder)
-                $this->prop['default_imap_folders'][$n] = rcube_charset_convert($folder, RCMAIL_CHARSET, 'UTF7-IMAP');
+        if (!empty($this->prop['default_folders']))
+            foreach ($this->prop['default_folders'] as $n => $folder)
+                $this->prop['default_folders'][$n] = rcube_charset_convert($folder, RCMAIL_CHARSET, 'UTF7-IMAP');
 
         // set PHP error logging according to config
         if ($this->prop['debug_level'] & 1) {
@@ -91,15 +106,20 @@ class rcube_config
                 ini_set('error_log', $this->prop['log_dir'].'/errors');
             }
         }
-        if ($this->prop['debug_level'] & 4) {
-            ini_set('display_errors', 1);
+
+        // enable display_errors in 'show' level, but not for ajax requests
+        ini_set('display_errors', intval(empty($_REQUEST['_remote']) && ($this->prop['debug_level'] & 4)));
+
+        // set timezone auto settings values
+        if ($this->prop['timezone'] == 'auto') {
+          $this->prop['_timezone_value'] = $this->client_timezone();
         }
-        else {
-            ini_set('display_errors', 0);
+        else if (is_numeric($this->prop['timezone'])) {
+          $this->prop['timezone'] = timezone_name_from_abbr("", $this->prop['timezone'] * 3600, 0);
         }
 
-        // clear output buffer
-        ob_end_clean();
+        // remove deprecated properties
+        unset($this->prop['dst_active']);
 
         // export config data
         $GLOBALS['CONFIG'] = &$this->prop;
@@ -130,13 +150,17 @@ class rcube_config
      * Read configuration from a file
      * and merge with the already stored config values
      *
-     * @param string Full path to the config file to be loaded
+     * @param string $fpath Full path to the config file to be loaded
      * @return booelan True on success, false on failure
      */
     public function load_from_file($fpath)
     {
         if (is_file($fpath) && is_readable($fpath)) {
+            // use output buffering, we don't need any output here 
+            ob_start();
             include($fpath);
+            ob_end_clean();
+
             if (is_array($rcmail_config)) {
                 $this->prop = array_merge($this->prop, $rcmail_config, $this->userprefs);
                 return true;
@@ -150,21 +174,43 @@ class rcube_config
     /**
      * Getter for a specific config parameter
      *
-     * @param  string Parameter name
-     * @param  mixed  Default value if not set
+     * @param  string $name Parameter name
+     * @param  mixed  $def  Default value if not set
      * @return mixed  The requested config value
      */
     public function get($name, $def = null)
     {
-        return isset($this->prop[$name]) ? $this->prop[$name] : $def;
+        if (isset($this->prop[$name])) {
+            $result = $this->prop[$name];
+        }
+        else if (isset($this->legacy_props[$name])) {
+            return $this->get($this->legacy_props[$name], $def);
+        }
+        else {
+            $result = $def;
+        }
+
+        $rcmail = rcmail::get_instance();
+
+        if ($name == 'timezone' && isset($this->prop['_timezone_value']))
+            $result = $this->prop['_timezone_value'];
+
+        if (is_object($rcmail->plugins)) {
+            $plugin = $rcmail->plugins->exec_hook('config_get', array(
+                'name' => $name, 'default' => $def, 'result' => $result));
+
+            return $plugin['result'];
+        }
+
+        return $result;
     }
 
 
     /**
      * Setter for a config parameter
      *
-     * @param string Parameter name
-     * @param mixed  Parameter value
+     * @param string $name  Parameter name
+     * @param mixed  $value Parameter value
      */
     public function set($name, $value)
     {
@@ -175,7 +221,7 @@ class rcube_config
     /**
      * Override config options with the given values (eg. user prefs)
      *
-     * @param array Hash array with config props to merge over
+     * @param array $prefs Hash array with config props to merge over
      */
     public function merge($prefs)
     {
@@ -187,12 +233,36 @@ class rcube_config
      * Merge the given prefs over the current config
      * and make sure that they survive further merging.
      *
-     * @param array  Hash array with user prefs
+     * @param array $prefs Hash array with user prefs
      */
     public function set_user_prefs($prefs)
     {
+        // Honor the dont_override setting for any existing user preferences
+        $dont_override = $this->get('dont_override');
+        if (is_array($dont_override) && !empty($dont_override)) {
+            foreach ($dont_override as $key) {
+                unset($prefs[$key]);
+            }
+        }
+
+        // convert user's timezone into the new format
+        if (is_numeric($prefs['timezone'])) {
+            $prefs['timezone'] = timezone_name_from_abbr('', $prefs['timezone'] * 3600, 0);
+        }
+
+        // larry is the new default skin :-)
+        if ($prefs['skin'] == 'default')
+            $prefs['skin'] = 'larry';
+
         $this->userprefs = $prefs;
-        $this->prop = array_merge($this->prop, $prefs);
+        $this->prop      = array_merge($this->prop, $prefs);
+
+        // override timezone settings with client values
+        if ($this->prop['timezone'] == 'auto') {
+            $this->prop['_timezone_value'] = isset($_SESSION['timezone']) ? $this->client_timezone() : $this->prop['_timezone_value'];
+        }
+        else if (isset($this->prop['_timezone_value']))
+           unset($this->prop['_timezone_value']);
     }
 
 
@@ -206,11 +276,30 @@ class rcube_config
         return $this->prop;
     }
 
+    /**
+     * Special getter for user's timezone offset including DST
+     *
+     * @return float  Timezone offset (in hours)
+     * @deprecated
+     */
+    public function get_timezone()
+    {
+      if ($tz = $this->get('timezone')) {
+        try {
+          $tz = new DateTimeZone($tz);
+          return $tz->getOffset(new DateTime('now')) / 3600;
+        }
+        catch (Exception $e) {
+        }
+      }
+
+      return 0;
+    }
 
     /**
      * Return requested DES crypto key.
      *
-     * @param string Crypto key name
+     * @param string $key Crypto key name
      * @return string Crypto key
      */
     public function get_crypto_key($key)
@@ -274,10 +363,11 @@ class rcube_config
     /**
      * Return the mail domain configured for the given host
      *
-     * @param string IMAP host
+     * @param string  $host   IMAP host
+     * @param boolean $encode If true, domain name will be converted to IDN ASCII
      * @return string Resolved SMTP host
      */
-    public function mail_domain($host)
+    public function mail_domain($host, $encode=true)
     {
         $domain = $host;
 
@@ -288,6 +378,9 @@ class rcube_config
         else if (!empty($this->prop['mail_domain']))
             $domain = rcube_parse_host($this->prop['mail_domain']);
 
+        if ($encode)
+            $domain = rcube_idn_to_ascii($domain);
+
         return $domain;
     }
 
@@ -302,4 +395,13 @@ class rcube_config
         return empty($this->errors) ? false : join("\n", $this->errors);
     }
 
+
+    /**
+     * Internal getter for client's (browser) timezone identifier
+     */
+    private function client_timezone()
+    {
+        return isset($_SESSION['timezone']) ? timezone_name_from_abbr("", $_SESSION['timezone'] * 3600, 0) : date_default_timezone_get();
+    }
+
 }
